#ifndef DICTSETTINGS_H
#define DICTSETTINGS_H

#include "ui_settings.h"

class DictStyle;
class DictSelectionConf;
class DictWindow;

class DictConf
{
	friend class DictSettings;
	// TODO friend function loadConf() only
	friend class DictWindow;
	KSharedConfigPtr config;
	QString dicthost;

	enum UserAgent {
		Short, Long, Custom
	};
	UserAgent agent;
	QString client;

	bool corrections, headings, matchPage, autoMatch, reformat, hilite, filter;
	int sidebarSide, defaultSearch, autoStrategy, maxMatches, maxSuggestions;

public:
	DictConf();
	~DictConf();
	void setHost(const QString &host) { dicthost = host; }

	void read();
	QStringList server();
	QString host();
	int port();
	QStringList database();
	QString strategy();
	int selected();

	void write();
	void writeServer(const QStringList &hosts);
	void writeHost(const QStringList &database, const QString &strategy);
	void writeSelected(int selected);

	bool optMime();
	QString username();
	QString password();
	void writeUser(const QString &username, const QString &password);
	void deleteUser();

	int sidebar();
	QList<int> widths();
	int stack();
	bool splitter();

	void writeSidebar(int index);
	void writeWidths(QList<int> sizes);

	QFont font();
	void writeFont(const QFont &font);

	DictSelectionConf selection(void);
	void writeSelection(const DictSelectionConf &conf);

	QStringList groups();
	void writeGroups(const QStringList &groups);

	QString userAgent() const;
	QString userAgent(UserAgent) const;
};


class DictSettings : public KDialog, public Ui::Settings
{
public:
	DictSettings(QWidget *parent);

	void setHosts(const QStringList &hosts);
	QStringList hosts();

	void setFont(const QFont &font);
	QFont font();

	void setStyle(DictStyle *style);

	void setSelection(const DictSelectionConf &conf);
	DictSelectionConf selection(void);

	void readConf(const DictConf &conf);
	void writeConf(DictConf &conf);
};

#endif
